package org.apidesign.apifest08.currency;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertTrue;
import static junit.framework.Assert.fail;
import static org.apidesign.apifest08.test.Task1Test.CZK;
import static org.apidesign.apifest08.test.Task1Test.SKK;
import static org.apidesign.apifest08.test.Task1Test.USD;

import org.junit.Test;

public class CompositeConvertorTest {
	private static final ExtendedConvertor USD_CZK_CONVERTOR = ConvertorFactory.createConvertor(new MoneyImpl(1,USD), new MoneyImpl(17,CZK));
	private static final ExtendedConvertor SKK_CZK_CONVERTOR = ConvertorFactory.createConvertor(new MoneyImpl(100,SKK), new MoneyImpl(80,CZK));

	@Test
	public void testCompose()
	{
		ExtendedConvertor convertor = new CompositeConvertor(USD_CZK_CONVERTOR, SKK_CZK_CONVERTOR);
		assertTrue(convertor.isConversionSupported(CZK, SKK));
		assertTrue(convertor.isConversionSupported(CZK, USD));
		assertFalse(convertor.isConversionSupported(SKK, USD));
		assertEquals(new MoneyImpl(10,SKK), convertor.convert(new MoneyImpl(8,CZK), SKK));
		assertEquals(new MoneyImpl(2,USD), convertor.convert(new MoneyImpl(34,CZK), USD));
		try
		{
			convertor.convert(new MoneyImpl(34,SKK), USD);
			fail("Exception expected");
		}
		catch(IllegalArgumentException e)
		{
			assertTrue("Ok", true);
		}
	}
	@Test
	public void testEmpty()
	{
		ExtendedConvertor convertor = new CompositeConvertor();
		assertFalse(convertor.isConversionSupported(SKK, USD));
		try
		{
			convertor.convert(new MoneyImpl(34,SKK), USD);
			fail("Exception expected");
		}
		catch(IllegalArgumentException e)
		{
			assertTrue("Ok", true);
		}
	}
	@Test(expected=NullPointerException.class)
	public void testCreateNull()
	{
		new CompositeConvertor(USD_CZK_CONVERTOR, null);
	}
}
